@@ -24,6 +24,10 @@
 
 #include <stdio.h>
 
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -42,11 +46,16 @@
 
 #include <glib.h>
 #include <gdk/gdkx.h>
+#include <gtk/gtk.h>
+
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/dialogs.h>
 
 #include "xfdesktop-common.h"
 
+#ifndef O_BINARY
+#define O_BINARY  0
+#endif
+
 gboolean
 xfdesktop_backdrop_list_is_valid(const gchar *path)
 {
@@ -54,16 +63,16 @@ xfdesktop_backdrop_list_is_valid(const gchar *path)
     gchar buf[512];
     gint size;
     gboolean is_list = FALSE;
-    
+
     size = sizeof(LIST_TEXT);
-    
+
     if(!(fp = fopen (path, "r")))
         return FALSE;
-    
+
     if(fgets(buf, size, fp) && !strncmp(LIST_TEXT, buf, size - 1))
         is_list = TRUE;
     fclose(fp);
-    
+
     return is_list;
 }
 
@@ -72,9 +81,9 @@ xfdesktop_backdrop_list_load(const gchar *filename,
                              gint *n_items,
                              GError **error)
 {
-    gchar *contents = NULL, **files = NULL, *p, *q;
+    gchar *contents = NULL, **files = NULL;
     gsize length = 0;
-    gint arr_size = 10, count = 0;
+    gint i, items;
 
     g_return_val_if_fail(filename && (!error || !*error), NULL);
 
@@ -90,33 +99,26 @@ xfdesktop_backdrop_list_load(const gchar *filename,
         return NULL;
     }
 
-    /* i'd use g_strsplit() here, but then counting is slower.  we can
-     * also filter out blank lines */
-    files = g_malloc(sizeof(gchar *) * (arr_size+1));
-    p = contents + sizeof(LIST_TEXT);
-    while(p && *p) {
-        q = strstr(p, "\n");
-        if(q) {
-            if(p == q)  /* blank line */
-                continue;
-            *q = 0;
-        } else
-            q = contents + length;  /* assume no trailing '\n' at EOF */
+    items = 0;
+    files = g_strsplit(contents, "\n", -1);
 
-        if(count == arr_size) {
-            arr_size += 10;
-            files = g_realloc(files, sizeof(gchar *) * (arr_size+1));
+    /* Since the first line is the file identifier, we need to skip it.
+     * Additionally, we want to skip blank lines. */
+    for(i = 1; files[i] != NULL; i++) {
+        if(g_strcmp0(files[i], "") != 0) {
+            g_free(files[items]);
+            files[items] = g_strdup(files[i]);
+            DBG("files[items] %s", files[items]);
+            items++;
         }
-
-        files[count++] = g_strdup(p);
-        if(q != contents + length)
-            p = q + 1;
     }
-    files[count] = NULL;
-    files = g_realloc(files, sizeof(gchar *) * (count+1));
+    files[items+1] = NULL;
 
+    files = g_realloc(files, sizeof(gchar *) * (items+1));
+
+    DBG("items %d", items);
     if(n_items)
-        *n_items = count;
+        *n_items = items;
 
     g_free(contents);
 
@@ -150,20 +152,20 @@ xfdesktop_backdrop_list_save(const gchar *filename,
                 if(error) {
                     g_set_error(error, G_FILE_ERROR,
                                 g_file_error_from_errno(errno),
-                                g_strerror(errno));
+                                "%s", g_strerror(errno));
                 }
                 unlink(filename_new);
             }
         } else {
             if(error) {
                 g_set_error(error, G_FILE_ERROR, g_file_error_from_errno(errno),
-                            g_strerror(errno));
+                            "%s", g_strerror(errno));
             }
             unlink(filename_new);
         }
     } else if(error) {
         g_set_error(error, G_FILE_ERROR, g_file_error_from_errno(errno),
-                    g_strerror(errno));
+                    "%s", g_strerror(errno));
     }
 
     g_free(filename_new);
@@ -178,7 +180,7 @@ xfdesktop_backdrop_list_choose_random(const gchar *filename,
     static gboolean __initialized = FALSE;
     static gint previndex = -1;
     gchar **files, *file = NULL;
-    gint n_items = 0, cur_file, i;
+    gint n_items = 0, cur_file, i, tries = 0;
 
     g_return_val_if_fail(filename && (!error || !*error), NULL);
 
@@ -211,8 +213,16 @@ xfdesktop_backdrop_list_choose_random(const gchar *filename,
 #endif
         __initialized = TRUE;
     }
-    
+
     do {
+        if(tries++ == n_items) {
+            /* this isn't precise, but if we've failed to get a good
+             * image after all this time, let's just give up */
+            g_warning("Unable to find good image from list; giving up");
+            g_strfreev(files);
+            return NULL;
+        }
+
         do {
 #ifdef HAVE_SRANDOM
             cur_file = random() % n_items;
@@ -220,6 +230,7 @@ xfdesktop_backdrop_list_choose_random(const gchar *filename,
             cur_file = rand() % n_items;
 #endif
         } while(cur_file == previndex && G_LIKELY(previndex != -1));
+
     } while(!xfdesktop_image_file_is_valid(files[cur_file]));
 
     previndex = cur_file;
@@ -240,7 +251,7 @@ pixbuf_loader_size_cb(GdkPixbufLoader *loader, gint width, gint height,
         gpointer user_data)
 {
     gboolean *size_read = user_data;
-    
+
     if(width > 0 && height > 0)
         *size_read = TRUE;
 }
@@ -249,34 +260,35 @@ gboolean
 xfdesktop_image_file_is_valid(const gchar *filename)
 {
     GdkPixbufLoader *loader;
-    FILE *fp;
+    int fd;
     gboolean size_read = FALSE;
     guchar buf[4096];
-    gint len;
+    gssize len;
 
     g_return_val_if_fail(filename, FALSE);
-    
-    fp = fopen(filename, "rb");
-    if(!fp)
+
+    fd = open(filename, O_RDONLY|O_BINARY);
+    if(fd < 0)
         return FALSE;
-    
+
     loader = gdk_pixbuf_loader_new();
     g_signal_connect(G_OBJECT(loader), "size-prepared",
             G_CALLBACK(pixbuf_loader_size_cb), &size_read);
-    
-    while(!feof(fp) && !ferror(fp)) {
-        if((len=fread(buf, 1, sizeof(buf), fp)) > 0) {
+
+    do {
+        len = read(fd, buf, sizeof(buf));
+        if(len > 0) {
             if(!gdk_pixbuf_loader_write(loader, buf, len, NULL))
                 break;
             if(size_read)
                 break;
         }
-    }
-    
-    fclose(fp);
+    } while(len > 0);
+
+    close(fd);
     gdk_pixbuf_loader_close(loader, NULL);
     g_object_unref(G_OBJECT(loader));
-    
+
     return size_read;
 }
 
@@ -288,7 +300,7 @@ xfdesktop_check_is_running(Window *xid)
     gint xscreen = -1;
     gchar selection_name[100];
     Atom selection_atom;
-    
+
     if(display) {
         if((p=g_strrstr(display, ".")))
             xscreen = atoi(p);
@@ -310,20 +322,20 @@ xfdesktop_send_client_message(Window xid, const gchar *msg)
 {
     GdkEventClient gev;
     GtkWidget *win;
-    
+
     win = gtk_invisible_new();
     gtk_widget_realize(win);
-    
+
     gev.type = GDK_CLIENT_EVENT;
     gev.window = win->window;
     gev.send_event = TRUE;
     gev.message_type = gdk_atom_intern("STRING", FALSE);
     gev.data_format = 8;
     strcpy(gev.data.b, msg);
-    
+
     gdk_event_send_client_message((GdkEvent *)&gev, (GdkNativeWindow)xid);
     gdk_flush();
-    
+
     gtk_widget_destroy(win);
 }
 
@@ -351,7 +363,10 @@ xfdesktop_popup_grab_available (GdkWindow *win, guint32 timestamp)
     while ((i++ < 2500) && (grab_failed = ((g1 != GDK_GRAB_SUCCESS)
                 || (g2 != GDK_GRAB_SUCCESS))))
     {
-        TRACE ("grab not available yet, waiting... (%i)", i);
+        TRACE ("grab not available yet, mouse reason: %d, keyboard reason: %d, waiting... (%i)", g1, g2, i);
+        if(g1 == GDK_GRAB_INVALID_TIME || g2 == GDK_GRAB_INVALID_TIME)
+            break;
+
         g_usleep (100);
         if (g1 != GDK_GRAB_SUCCESS)
         {
