@@ -55,6 +55,9 @@
 #define PREVIEW_HEIGHT  48
 #define MAX_ASPECT_RATIO 3.0f
 
+#define SETTINGS_WINDOW_LAST_WIDTH           "/last/window-width"
+#define SETTINGS_WINDOW_LAST_HEIGHT          "/last/window-height"
+
 #define SHOW_DESKTOP_MENU_PROP               "/desktop-menu/show"
 #define DESKTOP_MENU_SHOW_ICONS_PROP         "/desktop-menu/show-icons"
 
@@ -565,6 +568,7 @@ xfdesktop_settings_dialog_create_load_list(AppearancePanel *panel)
         gtk_main_iteration();
 
     if(!xfdesktop_settings_ensure_backdrop_list(list_file, parent)) {
+        g_warning ("backdrop list is not valid");
         g_free(list_file);
         return NULL;
     }
@@ -657,6 +661,7 @@ xfdesktop_settings_dialog_populate_image_list(AppearancePanel *panel)
 
             for(i = 0; images[i]; ++i) {
                 GtkTreeIter *iter = xfdesktop_settings_image_treeview_add(GTK_TREE_MODEL(ls), images[i]);
+                DBG("adding images[i] %s", images[i]);
                 if(iter)
                     gtk_tree_iter_free(iter);
             }
@@ -743,10 +748,17 @@ xfdesktop_settings_dialog_populate_image_list(AppearancePanel *panel)
 
     /* generate previews of each image -- the new thread will own
      * the reference on the list store, so let's not unref it here */
+#if GLIB_CHECK_VERSION(2, 32, 0)
+    if(!g_thread_try_new("create_previews", xfdesktop_settings_create_all_previews, ls, NULL)) {
+        g_critical("Failed to spawn thread; backdrop previews will be unavailable.");
+        g_object_unref(G_OBJECT(ls));
+    }
+#else
     if(!g_thread_create(xfdesktop_settings_create_all_previews, ls, FALSE, NULL)) {
         g_critical("Failed to spawn thread; backdrop previews will be unavailable.");
         g_object_unref(G_OBJECT(ls));
     }
+#endif
 
     g_free(image_file);
 
@@ -824,7 +836,7 @@ cb_xfdesktop_spin_icon_size_changed(GtkSpinButton *button,
         timer_id = 0;
     }
 
-    timer_id = g_timeout_add(2000,
+    timer_id = g_timeout_add(500,
                              (GSourceFunc)xfdesktop_spin_icon_size_timer,
                              button);
 
@@ -937,6 +949,19 @@ add_file_button_clicked(GtkWidget *button,
         }
         g_slist_free(filenames);
 
+#if GLIB_CHECK_VERSION(2, 32, 0)
+        if(!pdata->iters
+           || !g_thread_try_new("some_previews", xfdesktop_settings_create_some_previews,
+                                pdata, NULL))
+        {
+            if(pdata->iters)
+                g_critical("Unable to create thread for single image preview.");
+            g_object_unref(G_OBJECT(pdata->model));
+            g_slist_foreach(pdata->iters, (GFunc)gtk_tree_iter_free, NULL);
+            g_slist_free(pdata->iters);
+            g_free(pdata);
+        }
+#else
         if(!pdata->iters
            || !g_thread_create(xfdesktop_settings_create_some_previews,
                                pdata, FALSE, NULL))
@@ -948,7 +973,7 @@ add_file_button_clicked(GtkWidget *button,
             g_slist_free(pdata->iters);
             g_free(pdata);
         }
-
+#endif
         if(panel->image_list_loaded) {
             xfdesktop_settings_save_backdrop_list(panel, model);
 
@@ -1250,6 +1275,19 @@ image_treeview_drag_data_received(GtkWidget *widget,
 
     file_added = !!pdata->iters;
 
+#if GLIB_CHECK_VERSION(2, 32, 0)
+    if(!pdata->iters
+       || !g_thread_try_new("some_previews", xfdesktop_settings_create_some_previews,
+                           pdata, NULL))
+    {
+        if(pdata->iters)
+            g_critical("Unable to create thread for single image preview.");
+        g_object_unref(G_OBJECT(pdata->model));
+        g_slist_foreach(pdata->iters, (GFunc)gtk_tree_iter_free, NULL);
+        g_slist_free(pdata->iters);
+        g_free(pdata);
+    }
+#else
     if(!pdata->iters
        || !g_thread_create(xfdesktop_settings_create_some_previews,
                            pdata, FALSE, NULL))
@@ -1261,6 +1299,7 @@ image_treeview_drag_data_received(GtkWidget *widget,
         g_slist_free(pdata->iters);
         g_free(pdata);
     }
+#endif
 
     gtk_drag_finish(context, file_added, FALSE, time_);
 
@@ -1637,12 +1676,27 @@ xfdesktop_settings_dialog_add_screens(GtkBuilder *main_gxml,
 }
 
 static void
-xfdesktop_settings_response(GtkWidget *dialog, gint response_id)
+xfdesktop_settings_response(GtkWidget *dialog, gint response_id, gpointer user_data)
 {
     if(response_id == GTK_RESPONSE_HELP)
         xfce_dialog_show_help(GTK_WINDOW(dialog), "xfdesktop", "preferences", NULL);
-    else
+    else {
+        XfconfChannel *channel = (XfconfChannel*) user_data;
+        GdkWindowState state;
+        gint width, height;
+
+        /* don't save the state for full-screen windows */
+        state = gdk_window_get_state(GTK_WIDGET(dialog)->window);
+
+        if ((state & (GDK_WINDOW_STATE_MAXIMIZED | GDK_WINDOW_STATE_FULLSCREEN)) == 0) {
+            /* save window size */
+            gtk_window_get_size(GTK_WINDOW(dialog), &width, &height);
+            xfconf_channel_set_int(channel, SETTINGS_WINDOW_LAST_WIDTH, width);
+            xfconf_channel_set_int(channel, SETTINGS_WINDOW_LAST_HEIGHT, height);
+        }
+
         gtk_main_quit();
+    }
 }
 
 static GdkNativeWindow opt_socket_id = 0;
@@ -1714,7 +1768,12 @@ main(int argc, char **argv)
     if(opt_socket_id == 0) {
         dialog = GTK_WIDGET(gtk_builder_get_object(gxml, "prefs_dialog"));
         g_signal_connect(dialog, "response",
-                         G_CALLBACK(xfdesktop_settings_response), NULL);
+                         G_CALLBACK(xfdesktop_settings_response),
+                         channel);
+        gtk_window_set_default_size
+            (GTK_WINDOW(dialog),
+             xfconf_channel_get_int(channel, SETTINGS_WINDOW_LAST_WIDTH, -1),
+             xfconf_channel_get_int(channel, SETTINGS_WINDOW_LAST_HEIGHT, -1));
         gtk_window_present(GTK_WINDOW (dialog));
 
         /* To prevent the settings dialog to be saved in the session */
