@@ -255,7 +255,9 @@ main(int argc, char **argv)
     xfce_textdomain(GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
     
 #if defined(ENABLE_FILE_ICONS) || defined(USE_DESKTOP_MENU)
+#if !GLIB_CHECK_VERSION(2, 32, 0)
     g_thread_init(NULL);
+#endif /* GLIB_CHECK_VERSION */
 #endif
 #ifdef ENABLE_FILE_ICONS
     dbus_g_thread_init();
@@ -335,8 +337,8 @@ main(int argc, char **argv)
 
         return (already_running ? 0 : 1);
     }
-    
-    g_print("%s[%d]: starting up\n", PACKAGE, getpid());
+
+    DBG ("starting up");
 
     sm_client = xfce_sm_client_get();
     xfce_sm_client_set_restart_style(sm_client, XFCE_SM_CLIENT_RESTART_IMMEDIATELY);
@@ -357,7 +359,7 @@ main(int argc, char **argv)
         g_error_free(error);
         error = NULL;
     } else
-        channel = xfconf_channel_new(XFDESKTOP_CHANNEL);
+        channel = xfconf_channel_get(XFDESKTOP_CHANNEL);
 
     nscreens = gdk_display_get_n_screens(gdpy);
     desktops = g_new0(GtkWidget *, nscreens);
