@@ -35,6 +35,8 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk-pixbuf/gdk-pixdata.h>
 
+#include <libxfce4util/libxfce4util.h> /* for DBG/TRACE */
+
 #include "xfce-backdrop.h"
 #include "xfce-desktop-enum-types.h"
 #include "xfdesktop-common.h"  /* for DEFAULT_BACKDROP */
@@ -52,6 +54,7 @@ static void xfce_backdrop_get_property(GObject *object,
                                        guint property_id,
                                        GValue *value,
                                        GParamSpec *pspec);
+static gboolean xfce_backdrop_timer(XfceBackdrop *backdrop);
 
 struct _XfceBackdropPriv
 {
@@ -65,14 +68,20 @@ struct _XfceBackdropPriv
     gboolean show_image;
     XfceBackdropImageStyle image_style;
     gchar *image_path;
+    gchar *backdrop_list;
     
     gint brightness;
     gdouble saturation;
+
+    gboolean cycle_backdrop;
+    guint cycle_timer;
+    guint cycle_timer_id;
 };
 
 enum
 {
     BACKDROP_CHANGED,
+    BACKDROP_CYCLE,
     LAST_SIGNAL,
 };
 
@@ -87,6 +96,8 @@ enum
     PROP_IMAGE_FILENAME,
     PROP_BRIGHTNESS,
     PROP_SATURATION,
+    PROP_BACKDROP_CYCLE_ENABLE,
+    PROP_BACKDROP_CYCLE_TIMER,
 };
 
 static guint backdrop_signals[LAST_SIGNAL] = { 0, };
@@ -240,6 +251,10 @@ xfce_backdrop_class_init(XfceBackdropClass *klass)
             G_OBJECT_CLASS_TYPE(gobject_class), G_SIGNAL_RUN_FIRST,
             G_STRUCT_OFFSET(XfceBackdropClass, changed), NULL, NULL,
             g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+    backdrop_signals[BACKDROP_CYCLE] = g_signal_new("cycle",
+            G_OBJECT_CLASS_TYPE(gobject_class), G_SIGNAL_RUN_FIRST,
+            G_STRUCT_OFFSET(XfceBackdropClass, cycle), NULL, NULL,
+            g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
 #define XFDESKTOP_PARAM_FLAGS  (G_PARAM_READWRITE \
                                 | G_PARAM_CONSTRUCT \
@@ -305,6 +320,20 @@ xfce_backdrop_class_init(XfceBackdropClass *klass)
                                                         -10.0, 10.0, 1.0,
                                                         XFDESKTOP_PARAM_FLAGS));
 
+    g_object_class_install_property(gobject_class, PROP_BACKDROP_CYCLE_ENABLE,
+                                    g_param_spec_boolean("backdrop-cycle-enable",
+                                                         "backdrop-cycle-enable",
+                                                         "backdrop-cycle-enable",
+                                                         FALSE,
+                                                         XFDESKTOP_PARAM_FLAGS));
+
+    g_object_class_install_property(gobject_class, PROP_BACKDROP_CYCLE_TIMER,
+                                    g_param_spec_uint("backdrop-cycle-timer",
+                                                     "backdrop-cycle-timer",
+                                                     "backdrop-cycle-timer",
+                                                     0, G_MAXUSHORT, 10,
+                                                     XFDESKTOP_PARAM_FLAGS));
+
 #undef XFDESKTOP_PARAM_FLAGS
 }
 
@@ -314,14 +343,16 @@ xfce_backdrop_init(XfceBackdrop *backdrop)
     backdrop->priv = G_TYPE_INSTANCE_GET_PRIVATE(backdrop, XFCE_TYPE_BACKDROP,
                                                  XfceBackdropPriv);
     backdrop->priv->show_image = TRUE;
+    backdrop->priv->cycle_timer_id = 0;
+    backdrop->priv->backdrop_list = NULL;
 
     /* color defaults */
-    backdrop->priv->color1.red = 0x0f0f;
-    backdrop->priv->color1.green = 0x1717;
-    backdrop->priv->color1.blue = 0x2121;
-    backdrop->priv->color2.red = 0x0f0f;
-    backdrop->priv->color2.green = 0x1717;
-    backdrop->priv->color2.blue = 0x2121;
+    backdrop->priv->color1.red = 0x1515;
+    backdrop->priv->color1.green = 0x2222;
+    backdrop->priv->color1.blue = 0x3333;
+    backdrop->priv->color2.red = 0x1515;
+    backdrop->priv->color2.green = 0x2222;
+    backdrop->priv->color2.blue = 0x3333;
 }
 
 static void
@@ -333,7 +364,17 @@ xfce_backdrop_finalize(GObject *object)
     
     if(backdrop->priv->image_path)
         g_free(backdrop->priv->image_path);
-    
+
+    if(backdrop->priv->cycle_timer_id != 0) {
+        g_source_remove(backdrop->priv->cycle_timer_id);
+        backdrop->priv->cycle_timer_id = 0;
+    }
+
+    if(backdrop->priv->backdrop_list != NULL) {
+        g_free(backdrop->priv->backdrop_list);
+        backdrop->priv->backdrop_list = NULL;
+    }
+
     G_OBJECT_CLASS(xfce_backdrop_parent_class)->finalize(object);
 }
 
@@ -384,6 +425,14 @@ xfce_backdrop_set_property(GObject *object,
             xfce_backdrop_set_saturation(backdrop, g_value_get_double(value));
             break;
 
+        case PROP_BACKDROP_CYCLE_ENABLE:
+            xfce_backdrop_set_cycle_backdrop(backdrop, g_value_get_boolean(value));
+            break;
+
+        case PROP_BACKDROP_CYCLE_TIMER:
+            xfce_backdrop_set_cycle_timer(backdrop, g_value_get_uint(value));
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
             break;
@@ -432,6 +481,14 @@ xfce_backdrop_get_property(GObject *object,
             g_value_set_double(value, xfce_backdrop_get_saturation(backdrop));
             break;
 
+        case PROP_BACKDROP_CYCLE_ENABLE:
+            g_value_set_boolean(value, xfce_backdrop_get_cycle_backdrop(backdrop));
+            break;
+
+        case PROP_BACKDROP_CYCLE_TIMER:
+            g_value_set_uint(value, xfce_backdrop_get_cycle_timer(backdrop));
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
             break;
@@ -700,6 +757,37 @@ xfce_backdrop_get_image_filename(XfceBackdrop *backdrop)
     return backdrop->priv->image_path;
 }
 
+/**
+ * xfce_backdrop_set_list:
+ * @backdrop: An #XfceBackdrop.
+ * @backdrop_list: A list of backdrop images.
+ *
+ * Sets the internal list of wallpaper images assigned to this backdrop.
+ * Frees any previous backdrop list before setting the new one.
+ * [Transfer full]
+ **/
+void
+xfce_backdrop_set_list(XfceBackdrop *backdrop,
+                       gchar *backdrop_list)
+{
+    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
+
+    if(backdrop->priv->backdrop_list != NULL) {
+        g_free(backdrop->priv->backdrop_list);
+        backdrop->priv->backdrop_list = NULL;
+    }
+
+    backdrop->priv->backdrop_list = backdrop_list;
+}
+
+G_CONST_RETURN gchar*
+xfce_backdrop_get_list(XfceBackdrop *backdrop)
+{
+    g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), NULL);
+
+    return backdrop->priv->backdrop_list;
+}
+
 /**
  * xfce_backdrop_set_brightness:
  * @backdrop: An #XfceBackdrop.
@@ -745,6 +833,84 @@ xfce_backdrop_get_saturation(XfceBackdrop *backdrop)
     return backdrop->priv->saturation;
 }
 
+static gboolean
+xfce_backdrop_timer(XfceBackdrop *backdrop)
+{
+    TRACE("entering");
+
+    g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), FALSE);
+
+    g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CYCLE], 0);
+
+    return TRUE;
+}
+
+/**
+ * xfce_backdrop_set_cycle_timer:
+ * @backdrop: An #XfceBackdrop.
+ * @cycle_timer: The amount of time, in minutes, to wait before changing the
+ *               background image.
+ *
+ * If cycle_backdrop is enabled then this function will change the backdrop
+ * image based on the cycle_timer, a value between 0 and G_MAXUSHORT.
+ * A value of 0 indicates that the backdrop should not be changed.
+ **/
+void
+xfce_backdrop_set_cycle_timer(XfceBackdrop *backdrop, guint cycle_timer)
+{
+    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
+
+    TRACE("entering, cycle_timer = %d", cycle_timer);
+
+    if(cycle_timer > G_MAXUSHORT)
+        cycle_timer = G_MAXUSHORT;
+
+    backdrop->priv->cycle_timer = cycle_timer;
+
+    if(backdrop->priv->cycle_timer_id != 0) {
+        g_source_remove(backdrop->priv->cycle_timer_id);
+        backdrop->priv->cycle_timer_id = 0;
+    }
+
+    if(backdrop->priv->cycle_timer != 0 &&
+       backdrop->priv->cycle_backdrop == TRUE) {
+        backdrop->priv->cycle_timer_id = g_timeout_add_seconds(backdrop->priv->cycle_timer * 60,
+                                                               (GSourceFunc)xfce_backdrop_timer,
+                                                               backdrop);
+    }
+}
+
+guint
+xfce_backdrop_get_cycle_timer(XfceBackdrop *backdrop)
+{
+    g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), 0);
+    return backdrop->priv->cycle_timer;
+}
+
+void
+xfce_backdrop_set_cycle_backdrop(XfceBackdrop *backdrop,
+                                 gboolean cycle_backdrop)
+{
+    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
+
+    TRACE("entering, cycle_backdrop ? %s", cycle_backdrop == TRUE ? "TRUE" : "FALSE");
+
+    if(backdrop->priv->cycle_backdrop != cycle_backdrop) {
+        backdrop->priv->cycle_backdrop = cycle_backdrop;
+        /* Start or stop the backdrop changing */
+        xfce_backdrop_set_cycle_timer(backdrop,
+                                      xfce_backdrop_get_cycle_timer(backdrop));
+    }
+}
+
+gboolean
+xfce_backdrop_get_cycle_backdrop(XfceBackdrop *backdrop)
+{
+    g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), 0);
+
+    return backdrop->priv->cycle_backdrop;
+}
+
 /**
  * xfce_backdrop_get_pixbuf:
  * @backdrop: An #XfceBackdrop.
@@ -758,6 +924,8 @@ GdkPixbuf *
 xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
 {
     GdkPixbuf *final_image, *image = NULL, *tmp;
+    GdkPixbufFormat *format = NULL;
+    gboolean apply_backdrop_image = FALSE;
     gint i, j;
     gint w, h, iw = 0, ih = 0;
     XfceBackdropImageStyle istyle;
@@ -768,16 +936,13 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
     g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), NULL);
     
     if(backdrop->priv->show_image && backdrop->priv->image_path) {
-        image = gdk_pixbuf_new_from_file(backdrop->priv->image_path, NULL);
-        if(image) {
-            iw = gdk_pixbuf_get_width(image);
-            ih = gdk_pixbuf_get_height(image);
-        }
+        format = gdk_pixbuf_get_file_info(backdrop->priv->image_path, &iw, &ih);
+        /* make sure we have a usable backdrop image */
+        if(format != NULL)
+            apply_backdrop_image = TRUE;
     }
-    
+
     if(backdrop->priv->width == 0 || backdrop->priv->height == 0) {
-        if(!image)
-            return NULL;
         w = iw;
         h = ih;
     } else {
@@ -797,7 +962,7 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
             final_image = create_solid(&backdrop->priv->color1, w, h, FALSE, 0xff);
     }
     
-    if(!image) {
+    if(!apply_backdrop_image) {
         if(backdrop->priv->brightness != 0)
             final_image = adjust_brightness(final_image, backdrop->priv->brightness);
         
@@ -835,6 +1000,7 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
     
     switch(istyle) {
         case XFCE_BACKDROP_IMAGE_CENTERED:
+            image = gdk_pixbuf_new_from_file(backdrop->priv->image_path, NULL);
             dx = MAX((w - iw) / 2, 0);
             dy = MAX((h - ih) / 2, 0);
             xo = MIN((w - iw) / 2, dx);
@@ -845,7 +1011,13 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
             break;
         
         case XFCE_BACKDROP_IMAGE_TILED:
+            image = gdk_pixbuf_new_from_file(backdrop->priv->image_path, NULL);
             tmp = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, w, h);
+            /* Now that the image has been loaded, recalculate the image
+             * size because gdk_pixbuf_get_file_info doesn't always return
+             * the correct size */
+            iw = gdk_pixbuf_get_width(image);
+            ih = gdk_pixbuf_get_height(image);
             for(i = 0; (i * iw) < w; i++) {
                 for(j = 0; (j * ih) < h; j++) {
                     gint newx = iw * i, newy = ih * j;
@@ -867,10 +1039,10 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
             break;
         
         case XFCE_BACKDROP_IMAGE_STRETCHED:
-            xscale = (gdouble)w / iw;
-            yscale = (gdouble)h / ih;
+            image = gdk_pixbuf_new_from_file_at_scale(
+                            backdrop->priv->image_path, w, h, FALSE, NULL);
             gdk_pixbuf_composite(image, final_image, 0, 0, w, h,
-                    0, 0, xscale, yscale, interp, 255);
+                    0, 0, 1, 1, interp, 255);
             break;
         
         case XFCE_BACKDROP_IMAGE_SCALED:
@@ -887,9 +1059,12 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
             }
             dx = xo;
             dy = yo;
-            
+
+            image = gdk_pixbuf_new_from_file_at_scale(
+                        backdrop->priv->image_path, iw * xscale,
+                        ih * yscale, TRUE, NULL);
             gdk_pixbuf_composite(image, final_image, dx, dy,
-                    iw * xscale, ih * yscale, xo, yo, xscale, yscale,
+                    iw * xscale, ih * yscale, xo, yo, 1, 1,
                     interp, 255);
             break;
         
@@ -905,8 +1080,12 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
                 xo = 0;
                 yo = (h - (ih * yscale)) * 0.5;
             }
+
+            image = gdk_pixbuf_new_from_file_at_scale(
+                                backdrop->priv->image_path, iw * xscale,
+                                ih * yscale, TRUE, NULL);
             gdk_pixbuf_composite(image, final_image, 0, 0,
-                    w, h, xo, yo, xscale, yscale, interp, 255);
+                    w, h, xo, yo, 1, 1, interp, 255);
             break;
         
         default:
