@@ -75,7 +75,8 @@ static GdkPixbuf *xfdesktop_special_file_icon_peek_pixbuf(XfdesktopIcon *icon,
 static G_CONST_RETURN gchar *xfdesktop_special_file_icon_peek_label(XfdesktopIcon *icon);
 static G_CONST_RETURN gchar *xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon);
 static GdkDragAction xfdesktop_special_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon);
-static GdkDragAction xfdesktop_special_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon);
+static GdkDragAction xfdesktop_special_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
+                                                                          GdkDragAction *suggested_action);
 static gboolean xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
                                                          XfdesktopIcon *src_icon,
                                                          GdkDragAction action);
@@ -305,7 +306,8 @@ xfdesktop_special_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
 }
 
 static GdkDragAction
-xfdesktop_special_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon)
+xfdesktop_special_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
+                                                     GdkDragAction *suggested_action)
 {
     XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
     GFileInfo *info;
@@ -319,13 +321,20 @@ xfdesktop_special_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon)
             {
                 DBG("can move, copy and link");
                 actions = GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK;
+                if(suggested_action)
+                    *suggested_action = GDK_ACTION_MOVE;
             }
         }
     } else {
         DBG("can move");
         actions = GDK_ACTION_MOVE; /* everything else is just silly */
+        if(suggested_action)
+            *suggested_action = GDK_ACTION_MOVE;
     }
     
+    if(suggested_action)
+        *suggested_action = 0;
+
     return actions;
 }
 
@@ -344,7 +353,7 @@ xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
     DBG("entering");
     
     g_return_val_if_fail(special_file_icon && src_file_icon, FALSE);
-    g_return_val_if_fail(xfdesktop_special_file_icon_get_allowed_drop_actions(icon),
+    g_return_val_if_fail(xfdesktop_special_file_icon_get_allowed_drop_actions(icon, NULL),
                          FALSE);
     
     src_file = xfdesktop_file_icon_peek_file(src_file_icon);
@@ -439,7 +448,11 @@ xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon)
 
             size = g_file_info_get_attribute_uint64(info,
                                                     G_FILE_ATTRIBUTE_STANDARD_SIZE);
+#if GLIB_CHECK_VERSION (2, 30, 0)
+            size_string = g_format_size(size);
+#else
             size_string = g_format_size_for_display(size);
+#endif
 
             mtime = g_file_info_get_attribute_uint64(info,
                                                      G_FILE_ATTRIBUTE_TIME_MODIFIED);
@@ -562,6 +575,12 @@ xfdesktop_special_file_icon_changed(GFileMonitor *monitor,
     g_return_if_fail(G_IS_FILE(file));
     g_return_if_fail(XFDESKTOP_IS_SPECIAL_FILE_ICON(special_file_icon));
 
+    /* We don't care about change events only created/deleted */
+    if(event == G_FILE_MONITOR_EVENT_CHANGED ||
+       event == G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED ||
+       event == G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT)
+        return;
+
     /* release the old file information */
     if(special_file_icon->priv->file_info) {
         g_object_unref(special_file_icon->priv->file_info);
@@ -613,13 +632,6 @@ xfdesktop_special_file_icon_update_trash_count(XfdesktopSpecialFileIcon *special
         return;
     }
 
-    special_file_icon->priv->trash_item_count = g_file_info_get_attribute_uint32(
-                                                    special_file_icon->priv->file_info,
-                                                    G_FILE_ATTRIBUTE_TRASH_ITEM_COUNT);
-
-    if(special_file_icon->priv->trash_item_count == 0)
-        return;
-
     /* The trash count may return a number of files the user can't
      * currently delete, for example if the file is in a removable
      * drive that isn't mounted.
@@ -644,6 +656,7 @@ xfdesktop_special_file_icon_update_trash_count(XfdesktopSpecialFileIcon *special
     g_object_unref(enumerator);
 
     special_file_icon->priv->trash_item_count = n;
+    TRACE("exiting, trash count %d", n);
 }
 
 /* public API */
