@@ -81,7 +81,8 @@ static GdkPixbuf *xfdesktop_volume_icon_peek_pixbuf(XfdesktopIcon *icon,
 static G_CONST_RETURN gchar *xfdesktop_volume_icon_peek_label(XfdesktopIcon *icon);
 static G_CONST_RETURN gchar *xfdesktop_volume_icon_peek_tooltip(XfdesktopIcon *icon);
 static GdkDragAction xfdesktop_volume_icon_get_allowed_drag_actions(XfdesktopIcon *icon);
-static GdkDragAction xfdesktop_volume_icon_get_allowed_drop_actions(XfdesktopIcon *icon);
+static GdkDragAction xfdesktop_volume_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
+                                                                    GdkDragAction *suggested_action);
 static gboolean xfdesktop_volume_icon_do_drop_dest(XfdesktopIcon *icon,
                                                    XfdesktopIcon *src_icon,
                                                    GdkDragAction action);
@@ -328,7 +329,8 @@ xfdesktop_volume_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
 }
 
 static GdkDragAction
-xfdesktop_volume_icon_get_allowed_drop_actions(XfdesktopIcon *icon)
+xfdesktop_volume_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
+                                               GdkDragAction *suggested_action)
 {
     /* if not mounted, it doesn't really make sense to allow any operations
      * here.  if mounted, we should allow everything if it's writable. */
@@ -339,10 +341,16 @@ xfdesktop_volume_icon_get_allowed_drop_actions(XfdesktopIcon *icon)
     if(xfdesktop_volume_icon_is_mounted(icon)) {
         GFileInfo *info = xfdesktop_file_icon_peek_file_info(XFDESKTOP_FILE_ICON(icon));
         if(info) {
-            if(g_file_info_get_attribute_boolean(info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
+            if(g_file_info_get_attribute_boolean(info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE)) {
+                if(suggested_action)
+                    *suggested_action = GDK_ACTION_COPY;
                 return GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK;
+            }
         }
     }
+
+    if(suggested_action)
+        *suggested_action = 0;
     
     return 0;
 }
@@ -355,14 +363,14 @@ xfdesktop_volume_icon_do_drop_dest(XfdesktopIcon *icon,
     XfdesktopVolumeIcon *volume_icon = XFDESKTOP_VOLUME_ICON(icon);
     XfdesktopFileIcon *src_file_icon = XFDESKTOP_FILE_ICON(src_icon);
     GFileInfo *src_info;
-    GFile *src_file, *parent, *dest_file;
+    GFile *src_file, *parent, *dest_file = NULL;
     gboolean result = FALSE;
     gchar *name;
     
     DBG("entering");
     
     g_return_val_if_fail(volume_icon && src_file_icon, FALSE);
-    g_return_val_if_fail(xfdesktop_volume_icon_get_allowed_drop_actions(icon),
+    g_return_val_if_fail(xfdesktop_volume_icon_get_allowed_drop_actions(icon, NULL),
                          FALSE);
     
     src_file = xfdesktop_file_icon_peek_file(src_file_icon);
@@ -433,11 +441,15 @@ xfdesktop_volume_icon_peek_tooltip(XfdesktopIcon *icon)
 
             size = g_file_info_get_attribute_uint64(fs_info,
                                                     G_FILE_ATTRIBUTE_FILESYSTEM_SIZE);
-            size_string = g_format_size_for_display(size);
-
             free_space = g_file_info_get_attribute_uint64(fs_info,
                                                           G_FILE_ATTRIBUTE_FILESYSTEM_FREE);
+#if GLIB_CHECK_VERSION (2, 30, 0)
+            size_string = g_format_size(size);
+            free_space_string = g_format_size(free_space);
+#else
+            size_string = g_format_size_for_display(size);
             free_space_string = g_format_size_for_display(free_space);
+#endif
 
             volume_icon->priv->tooltip =
                 g_strdup_printf(_("Removable Volume\nMounted in \"%s\"\n%s left (%s total)"),
@@ -469,7 +481,7 @@ xfdesktop_volume_icon_eject_finish(GObject *object,
     g_return_if_fail(G_IS_ASYNC_RESULT(result));
     g_return_if_fail(XFDESKTOP_IS_VOLUME_ICON(icon));
 
-    if(!g_volume_eject_finish(volume, result, &error)) {
+    if(!g_volume_eject_with_operation_finish(volume, result, &error)) {
         /* ignore GIO errors handled internally */
         if(error->domain != G_IO_ERROR || error->code != G_IO_ERROR_FAILED_HANDLED) {
             gchar *volume_name = g_volume_get_name(volume);
@@ -511,7 +523,7 @@ xfdesktop_volume_icon_unmount_finish(GObject *object,
     g_return_if_fail(G_IS_ASYNC_RESULT(result));
     g_return_if_fail(XFDESKTOP_IS_VOLUME_ICON(icon));
 
-    if(!g_mount_unmount_finish(mount, result, &error)) {
+    if(!g_mount_unmount_with_operation_finish(mount, result, &error)) {
         /* ignore GIO errors handled internally */
         if(error->domain != G_IO_ERROR || error->code != G_IO_ERROR_FAILED_HANDLED) {
             gchar *mount_name = g_mount_get_name(mount);
@@ -627,18 +639,22 @@ xfdesktop_volume_icon_menu_toggle_mount(GtkWidget *widget,
 #ifdef HAVE_LIBNOTIFY
             xfdesktop_notify_eject(volume);
 #endif
-
-            g_volume_eject(volume, G_MOUNT_UNMOUNT_NONE, NULL,
-                           xfdesktop_volume_icon_eject_finish,
-                           g_object_ref(icon));
+            /* TODO: GMountOperation could be used to show what processes
+             *       are preventing an eject. */
+            g_volume_eject_with_operation(volume, G_MOUNT_UNMOUNT_NONE,
+                                          NULL, NULL,
+                                          xfdesktop_volume_icon_eject_finish,
+                                          g_object_ref(icon));
         } else {
 #ifdef HAVE_LIBNOTIFY
             xfdesktop_notify_unmount(mount);
 #endif
-
-            g_mount_unmount(mount, G_MOUNT_UNMOUNT_NONE, NULL,
-                            xfdesktop_volume_icon_unmount_finish, 
-                            g_object_ref(icon));
+            /* TODO: GMountOperation could be used to show what processes
+             *       are preventing an unmount. */
+            g_mount_unmount_with_operation(mount, G_MOUNT_UNMOUNT_NONE,
+                                           NULL, NULL,
+                                           xfdesktop_volume_icon_unmount_finish,
+                                           g_object_ref(icon));
         }
     } else {
         GMountOperation *operation;
@@ -676,7 +692,7 @@ xfdesktop_volume_icon_populate_context_menu(XfdesktopIcon *icon,
     GVolume *volume = volume_icon->priv->volume;
     GtkWidget *mi, *img;
     GMount *mount;
-    const gchar *icon_name, *icon_label;
+    const gchar *icon_name, *icon_label = NULL;
     
     img = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
     gtk_widget_show(img);
@@ -984,6 +1000,11 @@ xfdesktop_volume_icon_new(GVolume *volume,
         g_object_unref(mount);
     }
 
+    g_signal_connect_swapped(G_OBJECT(gtk_icon_theme_get_for_screen(screen)),
+                             "changed",
+                             G_CALLBACK(xfdesktop_volume_icon_invalidate_pixbuf),
+                             volume_icon);
+
     g_signal_connect(volume, "changed", 
                      G_CALLBACK(xfdesktop_volume_icon_changed), 
                      volume_icon);
